package test.com;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.GroupLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.jivesoftware.resource.Res;
import org.jivesoftware.resource.SparkRes;
import org.jivesoftware.smack.RosterEntry;
import org.jivesoftware.smack.packet.Presence;
import org.jivesoftware.smack.packet.RosterPacket;
import org.jivesoftware.spark.ChatManager;
import org.jivesoftware.spark.SparkManager;
import org.jivesoftware.spark.ui.ContactItem;
import org.jivesoftware.spark.util.log.Log;

public class TestContactItem extends ContactItem {
	private static String FONT_NAME = "Arial";
	private static int FONT_SIZE = 14;
	private static final Color NAME_COLOR = new Color(46, 85, 102);
	private static final int CUSTOM_VERTICAL_GAP = 0;
	private static final int ITEM_VERTICAL_GAP = 2;
	private static final Color BOTTOM_COLOR = new Color(143, 143, 143);
	private static final Color DESCRIPTION_COLOR = new Color(46, 85, 102);
	private final PicturePanel imgLabel;
	private final JLabel statusLabel;

	public TestContactItem(String alias, String nickname, String fullyQualifiedJID) {
		super(alias, nickname, fullyQualifiedJID, false);
		imgLabel = new PicturePanel();
		displayNameLabel = createLabel(NAME_COLOR, Font.BOLD);
		statusLabel = createLabel(BOTTOM_COLOR, Font.PLAIN);
		descriptionLabel = createLabel(DESCRIPTION_COLOR, Font.PLAIN);
		initComponents();
	}
	
	private JLabel createLabel(Color textColor, int fontType) {
		final JLabel label = new JLabel();
		label.setForeground(textColor);
		label.setFont(new Font(FONT_NAME, fontType, FONT_SIZE));

		return label;
	}
	
	private void updateAvatar() {
		try {
			final URL mugshotURL = getAvatarURL();
			ImageIcon mugshot = null;
			if (mugshotURL != null) {
				mugshot = new ImageIcon(mugshotURL);
			}

			if (mugshot == null || mugshot.getIconHeight() <= 1) {
				mugshot = SparkRes.getImageIcon(SparkRes.DEFAULT_AVATAR_64x64_IMAGE);
			}
			imgLabel.setIcon(mugshot);
		} catch (final MalformedURLException e) {
			Log.error(e);
		}
	}
	
	private void initComponents() {
		updateAvatar();
		final GroupLayout jPanel1Layout = new GroupLayout(imgLabel);
		imgLabel.setLayout(jPanel1Layout);
		jPanel1Layout.setHorizontalGroup(jPanel1Layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGap(0, 78,
				Short.MAX_VALUE));
		jPanel1Layout.setVerticalGroup(jPanel1Layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGap(0, 78,
				Short.MAX_VALUE));

		final int minHeight = 17;
		final GroupLayout layout = new GroupLayout(this);
		this.setLayout(layout);
		layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(
				layout.createSequentialGroup()
						.addContainerGap()
						.addComponent(imgLabel, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE,
								GroupLayout.PREFERRED_SIZE)
						.addGap(18, 18, 18)
						.addGroup(
								layout.createParallelGroup(GroupLayout.Alignment.LEADING)
										.addComponent(displayNameLabel).addComponent(descriptionLabel)
										.addComponent(statusLabel)).addContainerGap(255, Short.MAX_VALUE)));
		layout.setVerticalGroup(layout.createParallelGroup(GroupLayout.Alignment.LEADING).addGroup(
				layout.createSequentialGroup()
						.addGap(ITEM_VERTICAL_GAP)
						.addGroup(
								layout.createParallelGroup(GroupLayout.Alignment.LEADING)
										.addGroup(
												layout.createSequentialGroup()
														.addGap(ITEM_VERTICAL_GAP)
														.addComponent(imgLabel, GroupLayout.PREFERRED_SIZE,
																GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE))
										.addGroup(
														layout.createSequentialGroup()
																.addComponent(displayNameLabel, minHeight,
																		GroupLayout.DEFAULT_SIZE,
																		GroupLayout.PREFERRED_SIZE)
																.addGap(CUSTOM_VERTICAL_GAP)
																.addComponent(descriptionLabel, minHeight,
																		GroupLayout.DEFAULT_SIZE,
																		GroupLayout.PREFERRED_SIZE)
																.addGap(CUSTOM_VERTICAL_GAP)
																.addComponent(statusLabel, minHeight,
																		GroupLayout.DEFAULT_SIZE,
																		GroupLayout.PREFERRED_SIZE)))
						.addGap(ITEM_VERTICAL_GAP)));
	}
	
	@Override
	public void setIcon(Icon icon) {
		// just override default behavior
	}

	@Override
	public void setSideIcon(Icon icon) {
		updateAvatar();
	}

	/**
	 * Shows that the user is coming online.
	 */
	@Override
	public void showUserComingOnline() {
		// Change Font
		getNicknameLabel().setForeground(new Color(255, 128, 0));
	}

	/**
	 * Shows that the user is going offline.
	 */
	@Override
	public void showUserGoingOfflineOnline() {
		// Change Font
		getNicknameLabel().setForeground(Color.red);
	}
	
	@Override
	public void updatePresenceIcon(Presence p) {
        ChatManager chatManager = SparkManager.getChatManager();
        boolean handled = chatManager.fireContactItemPresenceChanged(this, p);
        if (handled) {
            return;
        }

        String status = p.getStatus();
        if ((status == null || status.isEmpty())  && p.isAvailable()) {
            Presence.Mode mode = p.getMode();
			if (mode == Presence.Mode.available) {
				status = Res.getString("status.online");
			} else if (mode == Presence.Mode.away) {
				status = Res.getString("status.away");
			} else if (mode == Presence.Mode.chat) {
				status = Res.getString("status.free.to.chat");
			} else if (mode == Presence.Mode.dnd) {
				status = Res.getString("status.do.not.disturb");
			} else if (mode == Presence.Mode.xa) {
				status = Res.getString("status.extended.away");
			}
        } else if ((status == null || status.isEmpty()) && p.isAway()) {
        	status = Res.getString("status.away");
        } else if ((status == null || status.isEmpty()) && !p.isAvailable()) {
        	RosterEntry entry = SparkManager.getConnection().getRoster().getEntry(getJID());
			if (entry != null
					&& (entry.getType() == RosterPacket.ItemType.none || entry.getType() == RosterPacket.ItemType.from)
					&& RosterPacket.ItemStatus.SUBSCRIPTION_PENDING == entry.getStatus()) {
				status = Res.getString("status.pending");
			}
        }

		if (p.getType() == Presence.Type.available) {
			if (status == null || status.isEmpty()) {
				status = Res.getString("status.online");
			}
		} else {
			status = "Offline";
		}

		statusLabel.setText(status);
		displayNameLabel.setForeground(DESCRIPTION_COLOR);
		imgLabel.setPresence(p);
		setStatus(status);
	}	
}
